<?php

/*
 * iPay88 Gateway for WooCommerce
 *
 * Description: Allows you to use iPay88 e-Wallet Payment with the WooCommerce plugin.
 * Plugin URI: https://www.ipay88.co.id/
 * Author: iPay88 Indonesia
 * License: The MIT License (MIT)
 *
 */
 
class ipay88_online_credit_gateway extends ipay88_OC_Abstract 
{
	//editor: Alda Mustafri
	
    public $id 			= 'ipay88_online_credit';
    public $has_fields 	= true;
	
    protected $ipay88_payment_method_selected;

    public function __construct() 
	{
        parent::__construct();
		
        $this->icon 				= IPAY88_OC_PLUGIN_URL . '/assets/images/ipay88.png';
        $this->method_title 		= __('iPay88 Online Credit Payment', IPAY88_OC_TEXT_DOMAIN);
        $this->method_description 	= __('Allows you to use iPay88 Online Credit Payment with the WooCommerce plugin.', IPAY88_OC_TEXT_DOMAIN);
       
	    $this->title = isset($this->settings['title']) ? $this->settings['title'] : "iPay88 Online Credit Payment.";
        $this->notify_url = add_query_arg('wc-api', 'ipay88_online_credit_gateway', home_url('/'));
		
        // Our Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
        add_action('woocommerce_api_' . $this->id . '_gateway', array($this, 'ipn_response'));
    	
		$this->image_ext 	= 'png';
		$this->hash_amount 	= 0;
		$this->formatted_amount = 0;
		

		// Define user set variables
		$this->title					= $this->settings['title'];
		$this->description				= $this->settings['description'];
		$this->enabled					= isset( $this->settings['enabled'] ) ? $this->settings['enabled'] : 'no';
		$this->MerchantCode				= $this->settings['merchant_code'];
		$this->MerchantKey				= $this->settings['merchant_key'];
		$this->PaymentId				= "";
		$this->OwnerPhone = $this->settings['merchant_phone'];
	}

    //Initial ipay88 Configs
    function init_configs($isPingback = false) 
	{
        ipay88_OC_Config::getInstance()->set(array( 'merchant_code' => $this->settings['merchant_code'],
            									 'merchant_key' => $this->settings['merchant_key']));
    }

    function getConfigOptionByKey($key) 
	{
        return $this->settings[$key];
    }

    //@param $order
    function receipt_page($order) 
	{
        $this->init_configs();
        echo $redirect_form = $this->generate_ipay88_form($order);

		echo '<p>'. __( 'Thank you for your order, please click the button below to pay with iPay88.', IPAY88_OC_TEXT_DOMAIN ) .'</p>';
    }

    function dd($data) 
	{
        echo "<pre>";
        print_r($data);
        echo "</pre>";
    }

	/**
	 * Generate the sha1 control signature. <br/>
	 * Used in both the request and the response to validate the authenticity of the message.
	 *
	 * @global object $woocommerce
	 * @param array $params The request or response parameters
	 * @param bool $is_response Are the parameters from the response message
	 * @return string The sha1 generated generate_ipay88_form
	 **/
	private function generate_sha1_signature( $params, $is_response = true )
	{
		$string = '';
		if ( $is_response ) {
			$this->format_amount( str_replace( ',', '', $params['Amount'] ) );
			$string = $this->MerchantKey . $this->MerchantCode . $params['PaymentId'] . $params['RefNo'] . $this->hash_amount/100 . $params['Currency'] . $params['Status'];
		} else {
			$string = $this->MerchantKey . $this->MerchantCode . $params['RefNo'] . $this->hash_amount . $params['Currency'];
		}

		return base64_encode( $this->hex2bin( sha1( $string ) ) );
	}

	function hex2bin( $hexSource )
	{
		$bin = '';
		for ( $i = 0; $i < strlen( $hexSource ); $i = $i + 2 ) {
			$bin .= chr( hexdec( substr( $hexSource, $i, 2 ) ) );
		}

		return $bin;
	}
	
	/**
	 * Generate iPay88 form
	 **/
    function generate_ipay88_form($order_id) 
	{
		$order = new WC_Order( $order_id );
		$desc = '';
		$itemTransactions = array();
		
		if ( 0 < sizeof( $order->get_items() ) ) 
		{
			foreach ( $order->get_items() as $item_id => $item ) 
			{
				$product = $item->get_product();
				array_push($itemTransactions, array(
					'Id' => $item_id,
					'Name' => $item->get_name(),
					'Quantity' => $item->get_quantity(),
					'Amount' => $item->get_total(),
					'Type' => $product->get_type(),
					'ImageUrl' => get_permalink( $product->get_id() ),
					'ParentType' => 'SELLER',
					'ParentId' => $this->MerchantCode
				));
				if ( $item['qty'] ) 
				{
					$item_name = htmlspecialchars($item['name']);
					$desc .= $item['qty'] .' x '. $item_name . ', ';
				}
			}
			
			//Add the description
			$desc = substr($desc, 0, -2 );
		}

		
		$currency = get_woocommerce_currency();

		$responseUrl 	=
		WC_Compat_iPay88_OC::force_https(add_query_arg('wc-api', 'ipay88_online_credit_gateway', home_url('/')));

		$backendUrl 	=
		WC_Compat_iPay88_OC::force_https(add_query_arg('iPay88_response', 'backend', add_query_arg('wc-api', 'ipay88_online_credit_gateway', home_url('/'))));

		// Format the order total
		$this->format_amount( $order->get_total() );

		$refNo			= date_format($order->get_date_created(), "Ymd") . "-" . $order_id;
		$paymentId 		= $_GET['ipay88_payment_method_selected'];
		
		$billing = $order->get_data()['billing'];
		$shipping = $order->get_data()['shipping'];

		$ipay88_args = array(
			'APIVersion' => '2.0',
			'MerchantCode'	=> $this->MerchantCode,
			'RefNo'			=> $refNo,
			'Amount'		=> $this->formatted_amount,
			'Currency'		=> $currency,
			'ProdDesc'		=> $desc,
			'UserName'      => $billing['first_name'] .' '. $billing['last_name'],
			'UserEmail'		=> $billing['email'],
			'UserContact'	=> $billing['phone'] ? $billing['phone'] : $shipping['phone'],
			'ResponseURL'	=> $responseUrl,
			'BackendURL'	=> $backendUrl,
			'PaymentId'		=> $paymentId,
			'Remark' => "WooCommerce Plugin||$refNo"
		);
		
		//Add signature
		$signatureText = "||" . $this->MerchantKey . "||" . $this->MerchantCode . "||" . $ipay88_args["RefNo"] . "||" . $ipay88_args['Amount'] . "||" . $ipay88_args["Currency"] . "||";
		$ipay88Signature = new ipay88_OC_Signature($signatureText);
		$ipay88_args['Signature'] = $ipay88Signature->getSignature();

		//add shipping to item transaction
		array_push($itemTransactions, array(
			'Id' => 'shippingfee',
			'Name' => $order->get_shipping_method(),
			'Amount' => $order->get_shipping_total(),
			'Quantity' => 1,
			'ParentType' => 'SELLER',
			'ParentId' => $this->MerchantCode
		));

		$defaultCountry = get_option("woocommerce_default_country");
		$countryCode = "";
		$stateCode = "";

		if ($defaultCountry) {
			$defaultCountry = explode(':', $defaultCountry);
			$countryCode = $defaultCountry[0];
			$stateCode = $defaultCountry[1];
		}

		$sellers = array();
		array_push($sellers, array(
			'Id' => $this->MerchantCode,
			'Name' => get_option("blogname"),
			'SelleridNumber' => '',
			'Email' => get_option("woocommerce_stock_email_recipient"),
			'Url' => get_option("siteurl"),
			'Address' => array(
				'FirstName' => get_option("blogname"),
				'LastName' => '',
				'Address' => get_option("woocommerce_store_address"),
				'City' => get_option("woocommerce_store_city"),
				'State' => $stateCode,
				'PostalCode' => get_option("woocommerce_store_postcode"),
				'Phone' => $this->OwnerPhone,
				'CountryCode' => $countryCode
			)
		));
		$ipay88_args['Sellers'] = $sellers;
		$ipay88_args['ItemTransactions'] = $itemTransactions;
		$ipay88_args['ShippingAddress'] = array(
			'FirstName' => $shipping['first_name'],
			'LastName' => $shipping['last_name'],
			'Address' => $shipping['address_1']. " ".$shipping['address_1'],
			'City' => $shipping['city'],
			'State' => $shipping['state'],
			'PostalCode' => $shipping['postcode'],
			'Phone' => $shipping['phone'] ? $shipping['phone'] : $billing['phone'],
			'CountryCode' => $shipping['country']
		);
		$ipay88_args['BillingAddress'] = array(
			'FirstName' => $billing['first_name'],
			'LastName' => $billing['last_name'],
			'Address' => $billing['address_1']." ".$billing['address_1'],
			'City' => $billing['city'],
			'State' => $billing['state'],
			'PostalCode' => $billing['postcode'],
			'Phone' => $billing['phone'] ? $billing['phone'] : $shipping['phone'],
			'CountryCode' => $billing['country']
		);

		$isTest 		= $this->getConfigOptionByKey('test_mode');
		if($isTest == 'no')
		{
			$getwayUrl = ipay88_OC_Indonesia::URL_PAYMENT_PRODUCTION;
			$checkoutApi = ipay88_OC_Indonesia::URL_PAYMENTAPI_PRODUCTION;
		}		
		else
		{
			$getwayUrl = ipay88_OC_Indonesia::URL_PAYMENT_SANDBOX;
			$checkoutApi = ipay88_OC_Indonesia::URL_PAYMENTAPI_SANDBOX;
		}

		$ipay88_JSON = json_encode($ipay88_args, JSON_UNESCAPED_SLASHES);
		WC_iPay88_OC::add_debug_log("iPay88 payment payload: $ipay88_JSON");

		$options 	 = array(
			'http' 	=> array(
				'header'  	=> "Content-type: application/json\r\n",
				'method'  	=> 'POST',
				"timeout" => (float) 30.0,
				'content' 	=> $ipay88_JSON
			)
		);

		$context = stream_context_create($options);
		$result = file_get_contents($checkoutApi, false, $context);

		global $woocommerce;
		if ($result === FALSE) {
			WC_iPay88_OC::add_debug_log("iPay88 Payment Failed. Error Description: Payment Request Failed. Request: $checkoutApi");
			$order->add_order_note(__("Payment request for order $order_id to iPay88 Indonesia failed", IPAY88_OC_TEXT_DOMAIN));

			$woocommerce->cart->empty_cart();

			return wp_redirect(WC()->cart->get_checkout_url());
		} else {
			$ipay88response = json_decode($result, true);

			$status = $ipay88response['Code'];
			$error_response = $ipay88response['Message'];

			if ($status == '0') {
				WC_iPay88_OC::add_debug_log("iPay88 Payment Failed. Error Description: Payment Request Failed. Status: $status - $error_response");
				$order->add_order_note(__("Payment request for order $order_id to iPay88 Indonesia failed", IPAY88_OC_TEXT_DOMAIN));

				$woocommerce->cart->empty_cart();

				return wp_redirect(WC()->cart->get_checkout_url());
			} else {
				$checkoutData = array(
					'Signature' => $ipay88_args['Signature'],
					'CheckoutID' => $ipay88response['CheckoutID']
				);
		
				$wiget = new ipay88_OC_Wiget($checkoutData);
				$wiget->setAction($getwayUrl);
		
				return $wiget->generateRedirectForm();
			}
		}
    }

    // Process the order after payment is made
    // @param int $order_id
    // @return array
    function process_payment($order_id) 
	{
		$order = wc_get_order($order_id);
		
		$payment_method_selected 			= $_POST['ipay88_bank_payment_method'];
        $_SESSION['posted_payment_type']	= $_POST['ipay88_bank_payment_method'];

		$orderkey = !method_exists($order, 'get_order_key') ? $order->order_key : $order->get_order_key();
		$orderid = !method_exists($order, 'get_id') ? $order->id : $order->get_id();

        return array(	'result' 	=> 'success',
            			'redirect' 	=> add_query_arg('key', $orderkey, add_query_arg(array(
                        'order-pay' => $orderid,
                        'ipay88_payment_method_selected' => $payment_method_selected),
                    	$order->get_checkout_payment_url(true))));
    }

    // Check the response from ipay88's Servers
    function ipn_response() 
	{
		WC_iPay88_OC::add_debug_log("ipn_response " . print_r($_REQUEST, true));
        global $woocommerce;

        $refNo = $_POST['RefNo'];
		$splitReffNo = explode("-", $refNo);
		$ReffNo_Date	= $splitReffNo[0];
		$ReffNo_Number	= $splitReffNo[1];

		if (isset($_REQUEST['iPay88_response'])) {
			$data = json_decode( file_get_contents('php://input'), true);

			$refNo = $data['RefNo'];
			$splitReffNo = explode("-", $refNo);
			$ReffNo_Date	= $splitReffNo[0];
			$ReffNo_Number	= $splitReffNo[1];
			$order = wc_get_order($ReffNo_Number);

			$merchantKey = $this->getConfigOptionByKey('merchant_key');
			$transaction = new ipay88_OC_Transaction($data);

			$transaction->setMerchantKey($merchantKey);
			$transaction->setSignature($data['Signature']);

			$result = array(
				'Code' => '1',
				'Message' => array(
					'Indonesia' => 'Pembayaran diterima',
					'English' => 'Status Received'
				)
			);
			
			WC_iPay88_OC::add_debug_log("transaction " . print_r($transaction, true));

			if($transaction->isPaid()) 
			{
				if($order->get_status() == 'pending' || $order->get_status() == 'on-hold')
				{
					WC_iPay88_OC::add_debug_log("transaction isPaid " . print_r($transaction, true));
					$order->add_order_note(__('The Order has been Paid by iPay88 Payment Gateway with Online Credit. The customer has paid IDR ' . ($data['Amount']) . ' - Transaction Id: ' . $data['TransId'], IPAY88_OC_TEXT_DOMAIN));
				
					$order->payment_complete();
					$order->update_status('processing');
					$order->save();
				}

				if($order->get_status() == 'completed')
				{
					$woocommerce->cart->empty_cart();
				}

				if (isset($_REQUEST['iPay88_response'])) {
					die(json_encode($result));
				} else {
					status_header(200);
					wc_add_notice('Your Payment success.', 'success');
					wp_redirect($this->get_return_url($order));
				}
			} 
			
			elseif ($transaction->isWaitingPayment()) 
			{
				WC_iPay88_OC::add_debug_log("transaction isWaitingPayment " . print_r($transaction, true));
				$order->add_order_note(__('The Order is waiting payment via ipay88 Payment Gateway - Transaction Id: ' . $data['TransId'], IPAY88_OC_TEXT_DOMAIN));
				
				$order->update_status('on-hold');
				$order->save();

				if (isset($_REQUEST['iPay88_response'])) {
					die(json_encode($result));
				} else {
					wp_redirect($this->get_return_url($order));
				}
			} 
			
			elseif ($transaction->isPaymentinProgress()) 
			{
				WC_iPay88_OC::add_debug_log("transaction isPaymentinProgress " . print_r($transaction, true));
				$order->add_order_note(__('The Order is waiting payment via ipay88 Payment Gateway - Transaction Id: ' . $data['TransId'], IPAY88_OC_TEXT_DOMAIN));
				
				$order->update_status('on-hold');
				$order->save();

				if (isset($_REQUEST['iPay88_response'])) {
					die(json_encode($result));
				} else {
					wp_redirect($this->get_return_url($order));
				}
			} 
			
			
			else 
			{
				WC_iPay88_OC::add_debug_log("transaction ErrDesc " . print_r($transaction, true));
				global $woocommerce;
				wc_add_notice('Your Payment Failed. Please try again or use another payment option.', 'error');

				$order->cancel_order(__('Reason: ' . $data['ErrDesc'], IPAY88_OC_TEXT_DOMAIN));

				if (isset($_REQUEST['iPay88_response'])) {
					die(json_encode($result));
				} else {
					return wp_redirect(WC()->cart->get_checkout_url());
				}
			}
		} else {
			$order = wc_get_order($ReffNo_Number);

			if (!$order) 
			{
				die('The order is Invalid!');
			}

			if($order->get_status() == 'pending' || $order->get_status() == 'on-hold')
			{
				wp_redirect($this->get_return_url($order));
			}

			if($order->get_status() == 'processing')
			{
				wp_redirect($order->get_checkout_order_received_url());
			}
		}
    }
	
	// Format the two amounts we need.
	// One for hashing
	// One for request parameter
	// @param double $amount
	function format_amount($amount) 
	{
		if (is_numeric($amount)) 
		{
			$this->hash_amount 		= number_format($amount, 0, '', '');
			$this->formatted_amount = number_format($amount, 0, '', '');
		}
	}
}
