<?php

defined('ABSPATH') or exit();
 /*
 * Plugin Name: iPay88 Online Credit Code for WooCommerce v4.x 
 * Plugin URI: https://docs.ipay88.co.id/woocommerce
 * Description: Allows you to use iPay88 Online Credit Payment with the WooCommerce plugin.
 * Version:  1.0.0
 * Author: iPay88 Indonesia
 * Author URI: https://www.ipay88.co.id/
 * Text Domain: https://www.ipay88.co.id/
 * License: The MIT License (MIT)
 *
 */

define('IPAY88_OC_TEXT_DOMAIN', 'ipay88-online-credit-for-woocommerce');
define('IPAY88_OC_DEFAULT_SUCCESS_PINGBACK_VALUE', 'OK');
define('IPAY88_OC_ORDER_STATUS_PENDING', 'wc-pending');
define('IPAY88_OC_ORDER_STATUS_COMPLETED', 'wc-completed');
define('IPAY88_OC_ORDER_STATUS_PROCESSING', 'wc-processing');
define('IPAY88_OC_PLUGIN_PATH', plugin_dir_path(__FILE__));
define('IPAY88_OC_PLUGIN_URL', plugins_url('', __FILE__));


function load_ipay88_online_credit_payments() 
{
	if (!class_exists('WC_Payment_Gateway')) return; // Nothing happens here is WooCommerce is not loaded

    include(IPAY88_OC_PLUGIN_PATH . '/lib/ipay88-online-credit-php/Autoload.php');
    include(IPAY88_OC_PLUGIN_PATH . '/includes/class-ipay88-online-credit-abstract.php');
    include(IPAY88_OC_PLUGIN_PATH . '/includes/class-ipay88-online-credit-gateway.php');
    include(IPAY88_OC_PLUGIN_PATH . '/includes/class-ipay88-compat.php');

    function ipay88_online_credit_payments($methods) 
	{
        $methods[] = 'ipay88_online_credit_gateway';
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'ipay88_online_credit_payments');
}

add_action('plugins_loaded', 'load_ipay88_online_credit_payments', 0);

/**
 * Add ipay88_online_credit Scripts
 */
function ipay88_online_credit_scripts() 
{
    wp_register_script('online_credit_js', IPAY88_OC_PLUGIN_URL.'/assets/js/payment.js', array('jquery'), '1', true);
    wp_enqueue_script('online_credit_js');
}

add_action('wp_enqueue_scripts', 'ipay88_online_credit_scripts');

/**
 * Require the woocommerce plugin installed first
 */
function ipay88_oc_child_plugin_notice() 
{ ?>
	<div class="error">
    <p><?php echo __("Sorry, but iPay88 Online Credit Payment Plugin requires the Woocommerce plugin to be installed and active.", IPAY88_OC_TEXT_DOMAIN)?></p>
    </div>
<?php
}

function ipay88_oc_child_plugin_has_parent_plugin() 
{
	if (is_admin() && current_user_can('activate_plugins') && !is_plugin_active('woocommerce/woocommerce.php')) 
	{
        add_action('admin_notices', 'ipay88_oc_child_plugin_notice');

        deactivate_plugins(plugin_basename(__FILE__));
        if (isset($_GET['activate'])) 
		{
            unset($_GET['activate']);
        }
    }
}

add_action('admin_init', 'ipay88_oc_child_plugin_has_parent_plugin');

/**
 * Adds plugin page links
 *
 * @since 1.0.0
 * @param array $links all plugin links
 * @return array $links all plugin links + our custom links (i.e., "Settings")
 */
function wc_ipay88_oc_gateway_plugin_links( $links ) 
{
    $plugin_links = array(
        '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=ipay88_online_credit' ) . '">' . __( 'Setting', 'ipay88_online_credit' ) . '</a>'
    );
    return array_merge( $plugin_links, $links );
}

add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'wc_ipay88_oc_gateway_plugin_links' );


class WC_iPay88_OC
{
	/**
	 * Savely get POST variables
	 *
	 * @since 1.1
	 * @param string $name POST variable name
	 * @return string The variable value
	 */
	public static function get_post( $name ) {
		if ( isset( $_POST[ $name ] ) ) {
		    return $_POST[ $name ];
		}
		return null;
	}
	
  /**
	 * WC Logger object
	 * @var object
	 */
	private static $log;

	/**
	 * Load gateway files
	 *
	 * @since 1.1
	 */
	public function load_gateway_files() {
		include_once('includes/class-wc-compat-ipay88.php');

	}

	/**
	 * Add debug log message
	 *
	 * @since 1.1
	 * @param string $message
	 */
	public static function add_debug_log( $message ) {
		if ( ! is_object( self::$log ) ) {
			self::$log = WC_Compat_iPay88_OC::get_wc_logger();
		}

		self::$log->add( 'ipay88', $message );
	}

	/**
	 * Savely get GET variables
	 *
	 * @since 1.1
	 * @param string $name GET variable name
	 * @return string The variable value
	 */
    public static function get_get( $name ) {
		if ( isset( $_GET[ $name ] ) ) {
		    return $_GET[ $name ];
		}
		return null;
	}


} new WC_iPay88_OC();